/* ld script to make i386 Linux kernel
 * Written by Martin Mares <mj@atrey.karlin.mff.cuni.cz>;
 */
OUTPUT_FORMAT("elf32-i386", "elf32-i386", "elf32-i386")
OUTPUT_ARCH(i386)
ENTRY(_start)

PHDRS {
	initdata PT_LOAD FLAGS(6);	/* RW_ */
	inittext PT_LOAD FLAGS(5);	/* R_E */
	text     PT_LOAD FLAGS(5);	/* R_E */
	rodata   PT_LOAD FLAGS(4);	/* R__ */
	data     PT_LOAD FLAGS(6);	/* RW_ */
}
SECTIONS
{
  . = __PAGE_OFFSET + 0x100000;
  .text.startup : {
	BYTE(0xEA) /* jmp far */
	LONG(startup_32 + __KERNEL_TEXT_OFFSET - __PAGE_OFFSET)
	SHORT(__KERNEL_CS)
	} :initdata

  . = ALIGN(4096);		/* Init code and data */
  __init_begin = .;
  .data.init : { *(.data.init) }
  . = ALIGN(16);
  __setup_start = .;
  .setup.init : { *(.setup.init) }
  __setup_end = .;
  __initcall_start = .;
  .initcall.init : { *(.initcall.init) }
  __initcall_end = .;

  _sinittext = . - __KERNEL_TEXT_OFFSET;

#ifdef CONFIG_PAX_KERNEXEC
  .text.init (. - __KERNEL_TEXT_OFFSET) : AT (_sinittext + __KERNEL_TEXT_OFFSET) {
	*(.text.init)
	_einittext = .;
	. = ALIGN(4*1024*1024) - 1;
	BYTE(0)
	} :inittext
  __init_end = . + __KERNEL_TEXT_OFFSET;

/*
 * PaX: this must be kept in synch with the KERNEL_CS base
 * in the GDTs in arch/i386/kernel/head.S
 */
  _text = .;			/* Text and read-only data */
  .text : AT (. + __KERNEL_TEXT_OFFSET) {
#else
  .text.init : { *(.text.init) } :inittext
  _einittext = .;
  . = ALIGN(4096);
  __init_end = .;
  _text = .;			/* Text and read-only data */
  .text : {
#endif

	*(.text)
	*(.fixup)
	*(.gnu.warning)
	} :text = 0x9090

  _etext = .;			/* End of text section */

  . = ALIGN(4096);
  . += __KERNEL_TEXT_OFFSET;
  .rodata.page_aligned : {
	*(.empty_zero_page)
	*(.pg0)

#ifdef CONFIG_X86_PAE
	*(.swapper_pm_dir)
#endif

	*(.swapper_pg_dir)
	*(.idt)
	} :rodata
  .rodata : { *(.rodata) *(.rodata.*) }
  .kstrtab : { *(.kstrtab) }

  . = ALIGN(16);		/* Exception table */
  __start___ex_table = .;
  __ex_table : { *(__ex_table) }
  __stop___ex_table = .;

  __start___ksymtab = .;	/* Kernel symbol table */
  __ksymtab : { *(__ksymtab) }
  __stop___ksymtab = .;

#ifdef CONFIG_PAX_KERNEXEC
  . = ALIGN(4*1024*1024);
#else
  . = ALIGN(32);
#endif

  _data = .;
  .data : {			/* Data */
	*(.data)
	CONSTRUCTORS
	} :data

  . = ALIGN(32);
  .data.cacheline_aligned : { *(.data.cacheline_aligned) }

  . = ALIGN(8192);
  .data.init_task : { *(.data.init_task) }

  . = ALIGN(4096);
  .data.page_aligned : {
	}

  _edata = .;			/* End of data section */

  __bss_start = .;		/* BSS */
  .bss : {
	*(.bss)
	}
  __bss_end = . ;

  _end = . ;

  /* Sections to be discarded */
  /DISCARD/ : {
	*(.text.exit)
	*(.data.exit)
	*(.exitcall.exit)
	}

  /* Stabs debugging sections.  */
  .stab 0 : { *(.stab) }
  .stabstr 0 : { *(.stabstr) }
  .stab.excl 0 : { *(.stab.excl) }
  .stab.exclstr 0 : { *(.stab.exclstr) }
  .stab.index 0 : { *(.stab.index) }
  .stab.indexstr 0 : { *(.stab.indexstr) }
  .comment 0 : { *(.comment) }
}
